document.addEventListener('DOMContentLoaded', function () {
    const canvas = document.getElementById('canvas');
    const moduleButtons = document.querySelectorAll('.sidebar .module-selector-btn');

    // Allow clicking modules in sidebar to add them
    moduleButtons.forEach(button => {
        button.addEventListener('click', function () {
            addModuleToCanvas(button.getAttribute('data-module'));
        });
        button.addEventListener('dragstart', handleDragStart);
    });

    // Drag & Drop Functionality
    canvas.addEventListener('dragover', handleDragOver);
    canvas.addEventListener('drop', handleDrop);

    function handleDragStart(e) {
        e.dataTransfer.setData('text/plain', e.target.getAttribute('data-module'));
        e.dataTransfer.effectAllowed = 'copy';
    }

    function handleDragOver(e) {
        e.preventDefault();
        e.dataTransfer.dropEffect = 'copy';
    }

    function handleDrop(e) {
        e.preventDefault();
        const moduleType = e.dataTransfer.getData('text/plain');
        addModuleToCanvas(moduleType, e.clientY);
    }

    // Function to Add Module to Canvas
    function addModuleToCanvas(moduleType, dropY = null) {
        if (!moduleMarkup[moduleType]) {
            console.warn(`No markup found for module type: ${moduleType}`);
            return;
        }

        const tempDiv = document.createElement('div');
        tempDiv.innerHTML = moduleMarkup[moduleType].trim();
        const moduleEl = tempDiv.firstElementChild;

        if (!moduleEl) {
            console.error(`Error creating module: ${moduleType}`);
            return;
        }

        moduleEl.classList.add('module-preview');

        const previewElement = moduleEl.querySelector('.module-preview');
        if (previewElement) previewElement.remove();

        const deleteIcon = document.createElement('div');
        deleteIcon.className = 'delete-icon';
        deleteIcon.innerHTML = '&times;';
        deleteIcon.onclick = function () {
            moduleEl.remove();
            updatePageNumbers();
        };
        moduleEl.appendChild(deleteIcon);

        if (dropY !== null) {
            let inserted = false;
            const modules = Array.from(canvas.querySelectorAll('.module-preview'));
            for (let mod of modules) {
                const rect = mod.getBoundingClientRect();
                if (dropY < rect.top + rect.height / 2) {
                    canvas.insertBefore(moduleEl, mod);
                    inserted = true;
                    break;
                }
            }
            if (!inserted) canvas.appendChild(moduleEl);
        } else {
            canvas.appendChild(moduleEl);
        }

        clearSelections();
        moduleEl.classList.add('selected');
        loadModuleSettings(moduleEl, moduleType);
        updatePageNumbers();
    }

    // Handle clicks to select modules
    canvas.addEventListener('click', function (e) {
        const clickedModule = e.target.closest('.module-preview');
        if (clickedModule) {
            clearSelections();
            clickedModule.classList.add('selected');
            const moduleType = clickedModule.getAttribute('data-module');
            loadModuleSettings(clickedModule, moduleType);
        }
    });

    function clearSelections() {
        document.querySelectorAll('.module-preview').forEach(m => m.classList.remove('selected'));
    }

    // ✅ MutationObserver for dynamic page numbering and button label
    const observer = new MutationObserver(updatePageNumbers);
    observer.observe(canvas, { childList: true, subtree: true });

    // Initial update
    updatePageNumbers();
});

/** ✅ Unified Dynamic Page Number and Button Label Update */
function updatePageNumbers() {
    const pageBreaks = document.querySelectorAll('.canvas .page-break-module');
    pageBreaks.forEach((module, index) => {
        const pageNumberSpan = module.querySelector('.dynamic-page-number');
        if (pageNumberSpan) {
            pageNumberSpan.textContent = index + 1;
        }
        const button = module.querySelector('[data-page-btn]');
        if (button) {
            button.textContent = (index === 0) ? 'Begin' : 'Next';
        }
    });
}
