let moduleCounter = 0; // Counter for unique module instances

/**
 * Adds a new module instance to the preview area.
 * @param {string} moduleFile - The module file name (e.g., "title_page.php")
 */
function addModule(moduleFile) {
    console.log(`Attempting to add module: ${moduleFile}`);
    const previewArea = document.getElementById("preview-area");
    moduleCounter++;
    let moduleId = `${moduleFile.replace('.php', '')}-${moduleCounter}`;

    fetch(`/modules/${moduleFile}`)
        .then(response => {
            if (!response.ok) throw new Error("Module not found");
            return response.text();
        })
        .then(html => {
            console.log(`Successfully fetched module: ${moduleFile}`);
            // Create container for this module instance.
            let moduleElement = document.createElement("div");
            moduleElement.classList.add("module");
            moduleElement.dataset.file = moduleFile;
            moduleElement.dataset.id = moduleId;
            moduleElement.innerHTML = html;

            // Create a Delete button.
            let deleteBtn = document.createElement("button");
            deleteBtn.textContent = "Delete";
            deleteBtn.classList.add("delete-btn");
            deleteBtn.style.marginTop = "10px";
            deleteBtn.addEventListener("click", function(e) {
                e.stopPropagation(); // Prevent module click.
                console.log(`Deleting module instance: ${moduleId}`);
                if (moduleElement.classList.contains("selected")) {
                    document.getElementById("settings-area").innerHTML = "<p>Select a module to adjust settings</p>";
                }
                moduleElement.remove();
                // Update page break numbers after deletion.
                if (typeof updatePageBreakNumbers === "function") {
                    updatePageBreakNumbers();
                }
            });
            moduleElement.appendChild(deleteBtn);

            // When this module instance is clicked, highlight it and load its settings.
            moduleElement.addEventListener("click", function () {
                document.querySelectorAll(".module").forEach(mod => mod.classList.remove("selected"));
                moduleElement.classList.add("selected");
                loadModuleSettings(moduleFile, moduleId);
            });

            previewArea.appendChild(moduleElement);

            // Explicitly mark this new module as selected and load its settings.
            document.querySelectorAll(".module").forEach(mod => mod.classList.remove("selected"));
            moduleElement.classList.add("selected");
            loadModuleSettings(moduleFile, moduleId);
            if (typeof updatePageBreakNumbers === "function") {
                updatePageBreakNumbers();
            }
        })
        .catch(error => console.error("Error loading module:", error));
}

/**
 * Loads settings for a module instance into the settings pane.
 * @param {string} moduleFile - The module file name.
 * @param {string} moduleId - The unique ID for this module instance.
 */
function loadModuleSettings(moduleFile, moduleId) {
    console.log(`Loading settings for: ${moduleFile} (Instance: ${moduleId})`);
    const settingsPane = document.getElementById("settings-area");

    // Get the module name from the module list.
    const moduleItem = document.querySelector(`.module-item[data-file="${moduleFile}"]`);
    const moduleName = moduleItem ? moduleItem.getAttribute("data-name") : moduleFile.replace(".php", "");

    // Set the settings title.
    settingsPane.innerHTML = `<h3>${moduleName} Settings (Instance: ${moduleId})</h3>`;

    // Use the external global moduleSettings object.
    if (moduleSettings && moduleSettings[moduleFile]) {
        let groupsRendered = {}; // To group fields if needed.
        moduleSettings[moduleFile].forEach(setting => {
            let container;
            if (setting.group) {
                if (!groupsRendered[setting.group]) {
                    container = document.createElement("div");
                    container.classList.add("settings-group");
                    let groupHeader = document.createElement("h4");
                    groupHeader.textContent = setting.group;
                    container.appendChild(groupHeader);
                    groupsRendered[setting.group] = container;
                    settingsPane.appendChild(container);
                } else {
                    container = groupsRendered[setting.group];
                }
            } else {
                container = settingsPane;
            }

            let fieldContainer = document.createElement("div");
            fieldContainer.classList.add("settings-field");

            let labelElem = document.createElement("label");
            labelElem.textContent = setting.label;
            fieldContainer.appendChild(labelElem);

            if (setting.type === "range" && setting.dualInput) {
                let rangeInput = document.createElement("input");
                rangeInput.type = "range";
                rangeInput.name = setting.name;
                rangeInput.min = setting.min;
                rangeInput.max = setting.max;
                rangeInput.step = setting.step;
                // Use defaultValue if provided; otherwise, use the midpoint.
                rangeInput.value = (setting.defaultValue !== undefined) ? setting.defaultValue : ((parseInt(setting.min) + parseInt(setting.max)) / 2);

                let numberInput = document.createElement("input");
                numberInput.type = "number";
                numberInput.name = setting.name + "_number";
                numberInput.min = setting.min;
                numberInput.max = setting.max;
                numberInput.step = setting.step;
                numberInput.value = rangeInput.value;

                rangeInput.addEventListener("input", () => {
                    numberInput.value = rangeInput.value;
                    updateModuleLive(setting.targetSelector, rangeInput.value, setting);
                });
                numberInput.addEventListener("input", () => {
                    rangeInput.value = numberInput.value;
                    updateModuleLive(setting.targetSelector, numberInput.value, setting);
                });

                fieldContainer.appendChild(rangeInput);
                fieldContainer.appendChild(numberInput);
            } else if (setting.type === "text" || setting.type === "number") {
                let inputField = document.createElement("input");
                inputField.type = setting.type;
                inputField.name = setting.name;
                inputField.value = setting.defaultValue || "";
                inputField.addEventListener("input", () => {
                    updateModuleLive(setting.targetSelector, inputField.value, setting);
                });
                fieldContainer.appendChild(inputField);
            } else if (setting.type === "file") {
                let fileInput = document.createElement("input");
                fileInput.type = "file";
                fileInput.name = setting.name;
                fieldContainer.appendChild(fileInput);

                let uploadBtn = document.createElement("button");
                uploadBtn.textContent = "Upload";
                uploadBtn.addEventListener("click", function() {
                    console.log("Upload button clicked for setting:", setting.name);
                    if (fileInput.files && fileInput.files[0]) {
                        const file = fileInput.files[0];
                        const formData = new FormData();
                        formData.append('logo', file);
                        fetch("/upload_logo", {
                            method: "POST",
                            body: formData
                        })
                        .then(response => response.json())
                        .then(data => {
                            if (data.url) {
                                updateModuleLive(setting.targetSelector, data.url, setting, true);
                            } else {
                                alert("Upload failed. Please try again.");
                            }
                        })
                        .catch(error => {
                            console.error("Error uploading file:", error);
                            alert("Error uploading file. Check console for details.");
                        });
                    } else {
                        alert("Please select a file first.");
                    }
                });
                fieldContainer.appendChild(uploadBtn);
            } else if (setting.type === "wysiwyg") {
                let textarea = document.createElement("textarea");
                textarea.name = setting.name;
                textarea.value = setting.defaultValue || "";
                fieldContainer.appendChild(textarea);

                // Initialize TinyMCE on this textarea after it's added.
                setTimeout(() => {
                    tinymce.init({
                        selector: `textarea[name="${setting.name}"]`,
                        height: 300,
                        menubar: false,
                        plugins: 'advlist autolink lists link charmap preview anchor',
                        toolbar: 'undo redo | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent',
                        content_css: '/js/tinymce/skins/content/default'
                    });
                }, 0);
            } else {
                let genericInput = document.createElement("input");
                genericInput.type = setting.type;
                genericInput.name = setting.name;
                genericInput.value = setting.defaultValue || "";
                genericInput.addEventListener("input", () => {
                    updateModuleLive(setting.targetSelector, genericInput.value, setting);
                });
                fieldContainer.appendChild(genericInput);
            }

            container.appendChild(fieldContainer);
        });
    } else {
        settingsPane.innerHTML += "<p>No settings available for this module.</p>";
    }
}


/**
 * Updates the currently selected module instance in the preview area live.
 * @param {string} targetSelector - CSS selector for the target element.
 * @param {string} value - The new value from the setting input.
 * @param {object} setting - The setting object.
 * @param {boolean} [isFile=false] - Whether this update is for a file input.
 */
function updateModuleLive(targetSelector, value, setting, isFile = false) {
    const selectedModule = document.querySelector(".module.selected");
    if (selectedModule) {
        let targetElement = selectedModule.querySelector(setting.targetSelector);
        console.log(`Updating ${setting.name}:`, targetElement, "with value:", value);
        if (targetElement) {
            if (isFile) {
                targetElement.src = value;
            } else {
                if (setting.type === "text") {
                    targetElement.textContent = value;
                } else if (setting.type === "range") {
                    targetElement.style.width = value + "%";
                }
            }
        } else {
            console.error("No target element found for selector", setting.targetSelector);
        }
        
        // Update the module's data-settings attribute.
        let currentSettings = {};
        try {
            currentSettings = JSON.parse(selectedModule.dataset.settings || '{}');
        } catch (e) {
            console.error("Error parsing module settings:", e);
            currentSettings = {};
        }
        currentSettings[setting.name] = value;
        selectedModule.dataset.settings = JSON.stringify(currentSettings);
    } else {
        console.error("No module selected for live update");
    }
}

// Attach event delegation for the left panel module list.
document.addEventListener("DOMContentLoaded", function() {
    const moduleList = document.getElementById("module-list");

    moduleList.addEventListener("click", function(event) {
        if (event.target && event.target.classList.contains("module-item")) {
            console.log(`Clicked module: ${event.target.dataset.file}`);
            addModule(event.target.dataset.file);
        }
    });

    // When clicking on the preview area (but not on a module), clear the settings pane.
    const previewArea = document.getElementById("preview-area");
    previewArea.addEventListener("click", function(event) {
        if (event.target === previewArea) {
            document.querySelectorAll(".module").forEach(mod => mod.classList.remove("selected"));
            document.getElementById("settings-area").innerHTML = "<p>Select a module to adjust settings</p>";
        }
    });
});

// jQuery UI Sortable, Draggable, Droppable setup.
$(document).ready(function() {
    $("#preview-area").sortable({
        placeholder: "ui-state-highlight",
        update: function(event, ui) {
            if (typeof updatePageBreakNumbers === "function") {
                updatePageBreakNumbers();
            }
        }
    });
    $(".module-item").draggable({
        helper: "clone",
        revert: "invalid"
    });
    $("#preview-area").droppable({
        accept: ".module-item",
        drop: function(event, ui) {
            let moduleFile = ui.draggable.data("file");
            addModule(moduleFile);
        }
    });
    // Reinitialize sortable for reordering.
    $("#preview-area").sortable({
        placeholder: "ui-state-highlight",
        update: function(event, ui) {
            if (typeof updatePageBreakNumbers === "function") {
                updatePageBreakNumbers();
            }
        }
    });
});

// Save App Functionality.
document.addEventListener("DOMContentLoaded", function() {
    document.getElementById("save-app-btn").addEventListener("click", function() {
        const appName = document.getElementById("app-name").value.trim();
        const appSlug = document.getElementById("app-slug").value.trim();
        const modules = [];
        document.querySelectorAll("#preview-area .module").forEach(moduleEl => {
            let settings = {};
            if (moduleEl.dataset.settings) {
                try {
                    settings = JSON.parse(moduleEl.dataset.settings);
                } catch (e) {
                    console.error("Error parsing settings for module " + moduleEl.dataset.id, e);
                }
            }
            modules.push({
                file: moduleEl.dataset.file,
                instanceId: moduleEl.dataset.id,
                settings: settings
            });
        });
        if (!appName || !appSlug) {
            alert("Please enter both the Application Name and URL Slug.");
            return;
        }
        const payload = {
            appName: appName,
            appSlug: appSlug,
            modules: modules
        };
        fetch("/save_app", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(payload)
        })
        .then(response => response.json())
        .then(data => {
            if (data.success) {
                alert("Application saved successfully!");
                window.location.href = "/apps/" + appSlug;
            } else {
                alert("Error saving application: " + data.message);
            }
        })
        .catch(error => {
            console.error("Error saving application:", error);
            alert("An error occurred while saving the application. Check console for details.");
        });
    });
});
